@@ -27,8 +27,8 @@
 #ifndef GTK_COLLECTION_H
 #define GTK_COLLECTION_H
 
+#include <GNUnet/gnunet_util.h>
+
 void fs_collection_start(void);
 
-void fs_collection_stop(void);
-
 #endif
