@@ -1,6 +1,6 @@
- /*
+/*
      This file is part of GNUnet.
-     (C) 2005 Christian Grothoff (and other contributing authors)
+     (C) 2005, 2006 Christian Grothoff (and other contributing authors)
 
      GNUnet is free software; you can redistribute it and/or modify
      it under the terms of the GNU General Public License as published
@@ -30,10 +30,11 @@
 #include "helper.h"
 #include "meta.h"
 #include "namespace.h"
+#include <GNUnet/gnunet_util_crypto.h>
+#include <GNUnet/gnunet_uritrack_lib.h>
+#include <GNUnet/gnunet_namespace_lib.h>
 #include <extractor.h>
 
-#define UPDATE_CONTENT_LIST_FREQUENCY (5 * cronMINUTES)
-
 /**
  * @brief linked list of pages in the search notebook
  */
@@ -54,21 +55,22 @@ static GladeXML * metaXML;
 
 static GtkWidget * makeNamespaceFrame(GtkWidget ** treeview,
 				      GtkWidget ** anonSpin) {
-  GtkWidget * window;
   GtkWidget * child;
   GtkWidget * resultList;
   GtkCellRenderer * renderer;
   GtkListStore * model;
   GladeXML * namespaceXML;
+  GtkTreeViewColumn * column;
   int col;
 
+  DEBUG_BEGIN();
   namespaceXML
     = glade_xml_new(getGladeFileName(),
 		    "namespaceContentFrame",
 		    PACKAGE_NAME);
   connectGladeWithPlugins(namespaceXML);
-  window = glade_xml_get_widget(namespaceXML,
-				"namespaceContentFrame");
+  child = extractMainWidgetFromWindow(namespaceXML,
+				      "namespaceContentFrame");
   resultList = glade_xml_get_widget(namespaceXML,
 				    "namespaceContentFrameTreeView");
   *anonSpin = glade_xml_get_widget(namespaceXML,
@@ -79,6 +81,7 @@ static GtkWidget * makeNamespaceFrame(GtkWidget ** treeview,
     gtk_list_store_new(IN_NAMESPACE_NUM,
 		       G_TYPE_STRING, /* (file)name */
 		       G_TYPE_UINT64,  /* size */
+		       G_TYPE_STRING,  /* human-readable size */
 		       G_TYPE_STRING, /* description */
 		       G_TYPE_STRING, /* mime-type */
 		       G_TYPE_STRING, /* last-ID */
@@ -89,6 +92,8 @@ static GtkWidget * makeNamespaceFrame(GtkWidget ** treeview,
 		       G_TYPE_POINTER);  /* META */
   gtk_tree_view_set_model(GTK_TREE_VIEW(resultList),
 			  GTK_TREE_MODEL(model));
+  gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(resultList)),
+			      GTK_SELECTION_MULTIPLE);
   renderer = gtk_cell_renderer_text_new();
   col = gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(resultList),
 					      -1,
@@ -96,16 +101,32 @@ static GtkWidget * makeNamespaceFrame(GtkWidget ** treeview,
 					      renderer,
 					      "text", IN_NAMESPACE_FILENAME,
 					      NULL);
+  column = gtk_tree_view_get_column(GTK_TREE_VIEW(resultList),
+				    col - 1);
+  gtk_tree_view_column_set_resizable(column, TRUE);
+  gtk_tree_view_column_set_clickable(column, TRUE);
+  gtk_tree_view_column_set_reorderable(column, TRUE);
+  gtk_tree_view_column_set_sort_column_id(column, IN_NAMESPACE_FILENAME);
+  /*gtk_tree_view_column_set_sort_indicator(column, TRUE);*/
+
   gtk_tree_view_column_set_resizable(gtk_tree_view_get_column(GTK_TREE_VIEW(resultList),
                                               col - 1),
                                               TRUE);
   renderer = gtk_cell_renderer_text_new();
+  g_object_set (renderer, "xalign", 1.00, NULL);
   col = gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(resultList),
 					      -1,
 					      _("Filesize"),
 					      renderer,
-					      "text", IN_NAMESPACE_SIZE,
+					      "text", IN_NAMESPACE_HSIZE,
 					      NULL);
+  column = gtk_tree_view_get_column(GTK_TREE_VIEW(resultList),
+				    col - 1);
+  gtk_tree_view_column_set_resizable(column, TRUE);
+  gtk_tree_view_column_set_clickable(column, TRUE);
+  gtk_tree_view_column_set_reorderable(column, TRUE);
+  gtk_tree_view_column_set_sort_column_id(column, IN_NAMESPACE_SIZE);
+  /*gtk_tree_view_column_set_sort_indicator(column, TRUE);*/
   gtk_tree_view_column_set_resizable(gtk_tree_view_get_column(GTK_TREE_VIEW(resultList),
                                               col - 1),
                                               TRUE);
@@ -116,6 +137,13 @@ static GtkWidget * makeNamespaceFrame(GtkWidget ** treeview,
 					      renderer,
 					      "text", IN_NAMESPACE_DESCRIPTION,
 					      NULL);
+  column = gtk_tree_view_get_column(GTK_TREE_VIEW(resultList),
+				    col - 1);
+  gtk_tree_view_column_set_resizable(column, TRUE);
+  gtk_tree_view_column_set_clickable(column, TRUE);
+  gtk_tree_view_column_set_reorderable(column, TRUE);
+  gtk_tree_view_column_set_sort_column_id(column, IN_NAMESPACE_DESCRIPTION);
+  /*gtk_tree_view_column_set_sort_indicator(column, TRUE);*/
   gtk_tree_view_column_set_resizable(gtk_tree_view_get_column(GTK_TREE_VIEW(resultList),
                                               col - 1),
                                               TRUE);
@@ -126,6 +154,13 @@ static GtkWidget * makeNamespaceFrame(GtkWidget ** treeview,
 					      renderer,
 					      "text", IN_NAMESPACE_MIMETYPE,
 					      NULL);
+  column = gtk_tree_view_get_column(GTK_TREE_VIEW(resultList),
+				    col - 1);
+  gtk_tree_view_column_set_resizable(column, TRUE);
+  gtk_tree_view_column_set_clickable(column, TRUE);
+  gtk_tree_view_column_set_reorderable(column, TRUE);
+  gtk_tree_view_column_set_sort_column_id(column, IN_NAMESPACE_MIMETYPE);
+  /*gtk_tree_view_column_set_sort_indicator(column, TRUE);*/
   gtk_tree_view_column_set_resizable(gtk_tree_view_get_column(GTK_TREE_VIEW(resultList),
                                               col - 1),
                                               TRUE);
@@ -136,6 +171,13 @@ static GtkWidget * makeNamespaceFrame(GtkWidget ** treeview,
 					      renderer,
 					      "text", IN_NAMESPACE_PUB_FREQ_STRING,
 					      NULL);
+  column = gtk_tree_view_get_column(GTK_TREE_VIEW(resultList),
+				    col - 1);
+  gtk_tree_view_column_set_resizable(column, TRUE);
+  gtk_tree_view_column_set_clickable(column, TRUE);
+  gtk_tree_view_column_set_reorderable(column, TRUE);
+  gtk_tree_view_column_set_sort_column_id(column, IN_NAMESPACE_PUB_FREQ_STRING);
+  /*gtk_tree_view_column_set_sort_indicator(column, TRUE);*/
   gtk_tree_view_column_set_resizable(gtk_tree_view_get_column(GTK_TREE_VIEW(resultList),
                                               col - 1),
                                               TRUE);
@@ -146,6 +188,13 @@ static GtkWidget * makeNamespaceFrame(GtkWidget ** treeview,
 					      renderer,
 					      "text", IN_NAMESPACE_PUB_DATE_STRING,
 					      NULL);
+  column = gtk_tree_view_get_column(GTK_TREE_VIEW(resultList),
+				    col - 1);
+  gtk_tree_view_column_set_reorderable(column, TRUE);
+  gtk_tree_view_column_set_resizable(column, TRUE);
+  gtk_tree_view_column_set_clickable(column, TRUE);
+  gtk_tree_view_column_set_sort_column_id(column, IN_NAMESPACE_PUB_DATE_STRING);
+  /*gtk_tree_view_column_set_sort_indicator(column, TRUE);*/
   gtk_tree_view_column_set_resizable(gtk_tree_view_get_column(GTK_TREE_VIEW(resultList),
                                               col - 1),
                                               TRUE);
@@ -156,6 +205,10 @@ static GtkWidget * makeNamespaceFrame(GtkWidget ** treeview,
 					      renderer,
 					      "text", IN_NAMESPACE_LAST_STRING,
 					      NULL);
+  column = gtk_tree_view_get_column(GTK_TREE_VIEW(resultList),
+				    col - 1);
+  gtk_tree_view_column_set_reorderable(column, TRUE);
+  gtk_tree_view_column_set_resizable(column, TRUE);
   gtk_tree_view_column_set_resizable(gtk_tree_view_get_column(GTK_TREE_VIEW(resultList),
                                               col - 1),
                                               TRUE);
@@ -166,15 +219,17 @@ static GtkWidget * makeNamespaceFrame(GtkWidget ** treeview,
 					      renderer,
 					      "text", IN_NAMESPACE_NEXT_STRING,
 					      NULL);
+  column = gtk_tree_view_get_column(GTK_TREE_VIEW(resultList),
+				    col - 1);
+  gtk_tree_view_column_set_reorderable(column, TRUE);
+  gtk_tree_view_column_set_resizable(column, TRUE);
   gtk_tree_view_column_set_resizable(gtk_tree_view_get_column(GTK_TREE_VIEW(resultList),
                                               col - 1),
                                               TRUE);
-  child = gtk_bin_get_child(GTK_BIN(window));
-  gtk_widget_ref(GTK_WIDGET(child));
-  gtk_container_remove(GTK_CONTAINER(window),
-		       child);
-  gtk_widget_destroy(window);
+
+				
   UNREF(namespaceXML);
+  DEBUG_END();
 
   return child;
 }
@@ -184,30 +239,21 @@ static GtkWidget * makeNamespaceFrame(GtkWidget ** treeview,
  * content model.  Check that it is not already
  * present!
  */
-static int updateView(const ECRS_FileInfo * fi,
-		      const HashCode512 * key,
-		      int isRoot,
-		      void * closure) {
-  GtkTreeModel * model = GTK_TREE_MODEL(closure);
-  struct ECRS_URI * euri;
+static void * updateView(void * cls) {
+  const ECRS_FileInfo * fi = cls;
   GtkTreeIter iter;
   char * filename;
   char * uriString;
   unsigned long long size;
+  char * size_h;
+  GtkWidget * contentList;
+  GtkTreeModel * model;
 
-  if (gtk_tree_model_get_iter_first(model,
-				    &iter)) {
-    do {	
-      gtk_tree_model_get(model,
-			 &iter,
-			 NAMESPACE_URI, &euri,
-			 -1);
-      if (ECRS_equalsUri(euri,
-			 fi->uri))
-	return OK; /* already listed */
-    } while (gtk_tree_model_iter_next(GTK_TREE_MODEL(model),
-				      &iter));
-  }
+  contentList
+    = glade_xml_get_widget(getMainXML(),
+			   "availableContentList");
+  model
+    = gtk_tree_view_get_model(GTK_TREE_VIEW(contentList));
   filename = ECRS_getFirstFromMetaData(fi->meta,
 				       EXTRACTOR_FILENAME,
 				       EXTRACTOR_TITLE,
@@ -220,15 +266,16 @@ static int updateView(const ECRS_FileInfo * fi,
 				       EXTRACTOR_PRODUCER,
 				       EXTRACTOR_UNKNOWN,
 				       -1);
-  if (filename == NULL)
+  if (filename == NULL) {
     filename = STRDUP(_("no name given"));
-  else {
-    char *dotdot;
-    
-    while ( NULL != (dotdot = strstr(filename, "..")))
+  } else {
+    char * dotdot;
+
+    while (NULL != (dotdot = strstr(filename, "..")))
       dotdot[0] = dotdot[1] = '_';
+    filename = validate_utf8(filename);
   }
-  
+
   if (ECRS_isFileUri(fi->uri))
     size = ECRS_fileSize(fi->uri);
   else
@@ -236,25 +283,43 @@ static int updateView(const ECRS_FileInfo * fi,
   uriString = ECRS_uriToString(fi->uri);
   gtk_list_store_append(GTK_LIST_STORE(model),
 			&iter);
+  size_h = string_get_fancy_byte_size(size);
   gtk_list_store_set(GTK_LIST_STORE(model),
 		     &iter,
 		     NAMESPACE_FILENAME, filename,
 		     NAMESPACE_SIZE, size,
+		     NAMESPACE_HSIZE, size_h,
 		     NAMESPACE_URISTRING, uriString,
 		     NAMESPACE_URI, ECRS_dupUri(fi->uri),
 		     NAMESPACE_META, ECRS_dupMetaData(fi->meta),
 		     -1);
+  FREE(size_h);
   FREE(filename);
   FREE(uriString);
+  return NULL;
+}
+
+/**
+ * Add the given content to the globally available
+ * content model.  Check that it is not already
+ * present!
+ */
+static int updateViewSave(const ECRS_FileInfo * fi,
+			  const HashCode512 * key,
+			  int isRoot,
+			  void * closure) {
+  gtkSaveCall(&updateView, (void*) fi);
   return OK;
 }
 
-static void clearContentList(void * mdl) {
+
+static void * clearContentList(void * mdl) {
   GtkTreeModel * model = GTK_TREE_MODEL(mdl);
   struct ECRS_URI * uri;
   struct ECRS_MetaData * meta;
   GtkTreeIter iter;
 
+  DEBUG_BEGIN();
   if (gtk_tree_model_get_iter_first(model,
 				    &iter)) {
     do {	
@@ -265,35 +330,16 @@ static void clearContentList(void * mdl) {
 			 -1);
       ECRS_freeUri(uri);
       ECRS_freeMetaData(meta);
-
+      gtk_list_store_set(GTK_LIST_STORE(model),
+			 &iter,
+			 NAMESPACE_URI, NULL,
+			 NAMESPACE_META, NULL,
+			 -1);
     } while (gtk_list_store_remove(GTK_LIST_STORE(model),
 				   &iter));
   }
-}
-
-/**
- * Update the list of the globally available content.
- */
-static void doUpdateContentList(void * unused) {
-  GtkWidget * contentList;
-  GtkTreeModel * model;
-
-  contentList
-    = glade_xml_get_widget(getMainXML(),
-			   "availableContentList");
-  model
-    = gtk_tree_view_get_model(GTK_TREE_VIEW(contentList));
-  FSUI_listURIs(&updateView,
-		model);
-}
-
-static void updateContentList(void * unused) {
-  gtkSaveCall(&doUpdateContentList, NULL);
-}
-
-void on_refreshContentListButton_clicked(gpointer dummy,
-					 gpointer dummy1) {
-  updateContentList(NULL);
+  DEBUG_END();
+  return NULL;
 }
 
 /**
@@ -325,7 +371,9 @@ static int addNamespaceContentToModel(void * cls,
   char * freq;
   char * date;
   unsigned long long size;
+  char * size_h;
 
+  DEBUG_BEGIN();
   filename = ECRS_getFirstFromMetaData(fi->meta,
 				       EXTRACTOR_FILENAME,
 				       EXTRACTOR_TITLE,
@@ -340,7 +388,7 @@ static int addNamespaceContentToModel(void * cls,
     filename = STRDUP(_("no name given"));
   else {
     char *dotdot;
-    
+
     while (NULL != (dotdot = strstr(filename, "..")))
       dotdot[0] = dotdot[1] = '_';
   }
@@ -378,12 +426,14 @@ static int addNamespaceContentToModel(void * cls,
     date = GN_CTIME(&nextPublicationTime);
 
   freq = updateIntervalToString(publicationFrequency);
+  size_h = string_get_fancy_byte_size(size);
   gtk_list_store_append(model,
 			&iter);
   gtk_list_store_set(model,
 		     &iter,
 		     IN_NAMESPACE_FILENAME, filename,
 		     IN_NAMESPACE_SIZE, size,
+		     IN_NAMESPACE_HSIZE, size_h,
 		     IN_NAMESPACE_DESCRIPTION, desc,
 		     IN_NAMESPACE_MIMETYPE, mime,
 		     IN_NAMESPACE_LAST_STRING, &last,
@@ -393,12 +443,13 @@ static int addNamespaceContentToModel(void * cls,
 		     IN_NAMESPACE_URI, ECRS_dupUri(fi->uri),
 		     IN_NAMESPACE_META, ECRS_dupMetaData(fi->meta),
 		     -1);
+  FREE(size_h);
   FREE(filename);
   FREE(uriString);
   FREE(freq);
   FREE(date);
   FREE(mime);
-
+  DEBUG_END();
   return OK;
 }
 
@@ -416,6 +467,7 @@ static int addTabForNamespace(void * unused,
   GtkWidget * notebook;
   GtkListStore * model;
 
+  DEBUG_BEGIN();
   label = gtk_label_new(namespaceName);
   list = MALLOC(sizeof(NamespaceList));
   list->name = STRDUP(namespaceName);
@@ -441,31 +493,33 @@ static int addTabForNamespace(void * unused,
 			   list->namespacepage,
 			   label);
   gtk_widget_show(notebook);
-  FSUI_listNamespaceContent
-    (ctx,
+  NS_listNamespaceContent
+    (ectx,
+     cfg,
      namespaceName,
      &addNamespaceContentToModel,
      model);
+  DEBUG_END();
   return OK;
 }
 
-void on_namespacemetaDataDialogKeywordAddButton_clicked(gpointer dummy,
-							GtkWidget * uploadButton) {
+void on_namespacemetaDataDialogKeywordAddButton_clicked_fs(gpointer dummy,
+							   GtkWidget * uploadButton) {
   handleKeywordListUpdate(metaXML,
 			  "namespaceKeywordEntry",
 			  "namespaceMetaDataDialogKeywordList");
 }
 
-void on_namespacemetaDataDialogMetaDataAddButton_clicked(gpointer dummy,
-							 GtkWidget * uploadButton) {
+void on_namespacemetaDataDialogMetaDataAddButton_clicked_fs(gpointer dummy,
+							    GtkWidget * uploadButton) {
   handleMetaDataListUpdate(metaXML,
 			   "namespaceMetaDataDialogMetaTypeComboBox",
 			   "namespaceMetaDataValueEntry",
 			   "namespaceMetaDataDialogMetaDataList");
 }
 
-void create_namespace_clicked(GtkWidget * dummy1,
-			      GtkWidget * dummy2) {
+void create_namespace_clicked_fs(GtkWidget * dummy1,
+				 GtkWidget * dummy2) {
   const char * namespaceName;
   GtkWidget * nameLine;
   GtkWidget * dialog;
@@ -476,6 +530,7 @@ void create_namespace_clicked(GtkWidget * dummy1,
   HashCode512 namespaceId;
   HashCode512 rootEntry;
 
+  DEBUG_BEGIN();
   metaXML
     = glade_xml_new(getGladeFileName(),
 		    "namespaceMetaDataDialog",
@@ -515,10 +570,13 @@ void create_namespace_clicked(GtkWidget * dummy1,
     nameLine = glade_xml_get_widget(metaXML,
 				    "namespaceNameEntry");
     namespaceName = gtk_entry_get_text(GTK_ENTRY(nameLine));
-    root = FSUI_createNamespace(ctx,
-				gtk_spin_button_get_value_as_int
-				(GTK_SPIN_BUTTON(spin)),
-				namespaceName,
+    root = NS_createNamespace(ectx,
+			      cfg,
+			      gtk_spin_button_get_value_as_int
+			      (GTK_SPIN_BUTTON(spin)),
+			      1000, /* FIXME: priority */
+			      999999, /* FIXME: expiration */
+			      namespaceName,
 				meta,
 				keywordURI,
 				&rootEntry);
@@ -553,10 +611,11 @@ void create_namespace_clicked(GtkWidget * dummy1,
   gtk_widget_destroy(dialog);
   UNREF(metaXML);
   metaXML = NULL;
+  DEBUG_END();
 }
 
-void namespaceDelete_clicked(GtkWidget * dummy1,
-			     GtkWidget * dummy2) {
+void namespaceDelete_clicked_fs(GtkWidget * dummy1,
+				GtkWidget * dummy2) {
   GtkWidget * notebook;
   NamespaceList * list;
   NamespaceList * prev;
@@ -565,6 +624,7 @@ void namespaceDelete_clicked(GtkWidget * dummy1,
   GtkWidget * dialog;
   gint ret;
 
+  DEBUG_BEGIN();
   notebook
     = glade_xml_get_widget(getMainXML(),
 			   "localNamespacesNotebook");
@@ -596,7 +656,7 @@ void namespaceDelete_clicked(GtkWidget * dummy1,
     list = list->next;
   }
   if (list == NULL) {
-    BREAK();
+    GE_BREAK(ectx, 0);
     return;
   }
   /* open window to ask for confirmation,
@@ -620,10 +680,13 @@ void namespaceDelete_clicked(GtkWidget * dummy1,
     head = list->next;
   else
     prev->next = list->next;
-  FSUI_deleteNamespace(list->name);
+  NS_deleteNamespace(ectx,
+		     cfg,
+		     list->name);
   FREE(list->name);
   ECRS_freeMetaData(list->meta);
   FREE(list);
+  DEBUG_END();
 }
 
 typedef struct {
@@ -652,6 +715,7 @@ static void initiateUpload(GtkTreeModel * model,
   NamespaceList * list;
   ECRS_FileInfo fi;
 
+  DEBUG_BEGIN();
   dst = NULL;
   meta = cls->meta;
   gtk_tree_model_get(model,
@@ -665,18 +729,21 @@ static void initiateUpload(GtkTreeModel * model,
      metadata immediately with the dialog. */
 
   if (dst == NULL) {
-    BREAK();
+    GE_BREAK(ectx, 0);
     return;
   }
-  resultURI = FSUI_addToNamespace(ctx,
-				  cls->anonymityLevel,
-				  cls->namespaceName,
-				  cls->updateInterval,
-				  cls->lastId,
-				  &cls->thisId,
-				  cls->nextId,
-				  dst,
-				  meta);
+  resultURI = NS_addToNamespace(ectx,
+				cfg,
+				cls->anonymityLevel,
+				1000, /* FIXME: priority */
+				999999, /* FIXME: expiration */
+				cls->namespaceName,
+				cls->updateInterval,
+				cls->lastId,
+				&cls->thisId,
+				cls->nextId,
+				dst,
+				meta);
   if (resultURI != NULL) {
     list = head;
     while ( (list != NULL) &&
@@ -684,7 +751,7 @@ static void initiateUpload(GtkTreeModel * model,
 			 list->name)) )
       list = list->next;
     if (list == NULL) {
-      BREAK();
+      GE_BREAK(ectx, 0);
     } else {
       /* update namespace content list! */
       fi.uri = dst;
@@ -702,18 +769,19 @@ static void initiateUpload(GtkTreeModel * model,
 		_("Failed to insert content into namespace "
 		  "(consult logs).\n"));
   }
+  DEBUG_END();
 }
 
-void on_namespaceInsertMetaDataDialogMetaDataAddButton_clicked(GtkWidget * dummy1,
-							       GtkWidget * dummy2) {
+void on_namespaceInsertMetaDataDialogMetaDataAddButton_clicked_fs(GtkWidget * dummy1,
+								  GtkWidget * dummy2) {
   handleMetaDataListUpdate(metaXML,
 			   "namespaceInsertMetaTypeComboBox",
 			   "metaDataValueEntry",
 			   "metaDataTreeView");
 }
 
-void on_namespaceInsertButton_clicked(GtkWidget * dummy1,
-				      GtkWidget * dummy2) {
+void on_namespaceInsertButton_clicked_fs(GtkWidget * dummy1,
+					 GtkWidget * dummy2) {
   const char * identifierName;
   NamespaceList * list;
   GtkWidget * nameLine;
@@ -728,6 +796,7 @@ void on_namespaceInsertButton_clicked(GtkWidget * dummy1,
   gint num;
   GtkTreeIter iter;
 
+  DEBUG_BEGIN();
   contentList
     = glade_xml_get_widget(getMainXML(),
 			   "availableContentList");
@@ -749,7 +818,7 @@ void on_namespaceInsertButton_clicked(GtkWidget * dummy1,
   if (FALSE == gtk_tree_selection_get_selected(selection,
 					       NULL,
 					       &iter)) {
-    BREAK();
+    GE_BREAK(ectx, 0);
     return;
   }
   gtk_tree_model_get(gtk_tree_view_get_model(GTK_TREE_VIEW(contentList)),
@@ -761,7 +830,7 @@ void on_namespaceInsertButton_clicked(GtkWidget * dummy1,
     = glade_xml_get_widget(getMainXML(),
 			   "localNamespacesNotebook");
   num = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
-  GNUNET_ASSERT(num != -1);
+  GE_ASSERT(ectx, num != -1);
   page =gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook),
 				  num);
   list = head;
@@ -769,7 +838,7 @@ void on_namespaceInsertButton_clicked(GtkWidget * dummy1,
 	  (list->namespacepage != page) )
     list = list->next;
   if (list == NULL) {
-    BREAK();
+    GE_BREAK(ectx, 0);
     return;
   }
   cls.namespaceName = list->name;
@@ -811,8 +880,8 @@ void on_namespaceInsertButton_clicked(GtkWidget * dummy1,
 			       "metaDataTreeView",
 			       "namespaceInsertPreview");
     cls.anonymityLevel
-      = getAnonymityLevel(metaXML,
-			  "anonymitySpinButton");
+      = getSpinButtonValue(metaXML,
+			   "anonymitySpinButton");
     nameLine = glade_xml_get_widget(metaXML,
 				    "namespaceContentIdentifierEntry");
     identifierName = gtk_entry_get_text(GTK_ENTRY(nameLine));
@@ -836,7 +905,7 @@ void on_namespaceInsertButton_clicked(GtkWidget * dummy1,
       cls.nextId = &nextId;
     }
     cls.meta = meta;
-    gtk_tree_selection_selected_foreach
+    ggc_tree_selection_selected_foreach
       (selection,
        &initiateUpload,
        &cls);
@@ -846,10 +915,11 @@ void on_namespaceInsertButton_clicked(GtkWidget * dummy1,
   gtk_widget_destroy(dialog);
   UNREF(metaXML);
   metaXML = NULL;
+  DEBUG_END();
 }
 
-void on_namespaceUpdateButton_clicked(GtkWidget * dummy1,
-				      GtkWidget * dummy2) {
+void on_namespaceUpdateButton_clicked_fs(GtkWidget * dummy1,
+					 GtkWidget * dummy2) {
   const char * identifierName;
   NamespaceList * list;
   GtkWidget * nameLine;
@@ -867,10 +937,11 @@ void on_namespaceUpdateButton_clicked(GtkWidget * dummy1,
   GtkTreeSelection * selectionNamespace;
   IUC cls;
   gint num;
-  const char * last;
-  const char * next;
-  const char * freq;
+  char * last;
+  char * next;
+  char * freq;
 
+  DEBUG_BEGIN();
   contentList
     = glade_xml_get_widget(getMainXML(),
 			   "availableContentList");
@@ -892,7 +963,7 @@ void on_namespaceUpdateButton_clicked(GtkWidget * dummy1,
   if (FALSE == gtk_tree_selection_get_selected(selection,
 					       NULL,
 					       &iter)) {
-    BREAK();
+    GE_BREAK(ectx, 0);
     return;
   }
   gtk_tree_model_get(gtk_tree_view_get_model(GTK_TREE_VIEW(contentList)),
@@ -904,7 +975,7 @@ void on_namespaceUpdateButton_clicked(GtkWidget * dummy1,
     = glade_xml_get_widget(getMainXML(),
 			   "localNamespacesNotebook");
   num = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
-  GNUNET_ASSERT(num != -1);
+  GE_ASSERT(ectx, num != -1);
   page =gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook),
 				  num);
   list = head;
@@ -912,7 +983,7 @@ void on_namespaceUpdateButton_clicked(GtkWidget * dummy1,
 	  (list->namespacepage != page) )
     list = list->next;
   if (list == NULL) {
-    BREAK();
+    GE_BREAK(ectx, 0);
     return;
   }
   cls.namespaceName = list->name;
@@ -936,7 +1007,7 @@ void on_namespaceUpdateButton_clicked(GtkWidget * dummy1,
   if (FALSE == gtk_tree_selection_get_selected(selectionNamespace,
 					       NULL,
 					       &iter)) {
-    BREAK();
+    GE_BREAK(ectx, 0);
     return;
   }
   gtk_tree_model_get(list->model,
@@ -957,23 +1028,29 @@ void on_namespaceUpdateButton_clicked(GtkWidget * dummy1,
 		     next);
   if (OK != enc2hash(next,
 		     &cls.thisId)) {
-    BREAK();
+    GE_BREAK(ectx, 0);
     UNREF(metaXML);
     metaXML = NULL;
+    if (last != NULL)
+      free(last);
+    if (next != NULL)
+      free(next);
+    if (freq != NULL)
+      free(freq);
     return;
   }
   if (OK == enc2hash(last,
 		     &prevId)) {
     cls.lastId = &prevId;
   } else {
-    BREAK(); /* should not happen, try to continue */
+    GE_BREAK(ectx, 0); /* should not happen, try to continue */
     cls.lastId = NULL;
   }
   nameLine = glade_xml_get_widget(metaXML,
 				  "nextIdentifierEntry");
   if (OK != parseTimeInterval(freq,
 			      &cls.updateInterval)) {
-    BREAK();
+    GE_BREAK(ectx, 0);
     cls.updateInterval = ECRS_SBLOCK_UPDATE_SPORADIC;
   }
   if (cls.updateInterval == ECRS_SBLOCK_UPDATE_SPORADIC) {
@@ -982,14 +1059,22 @@ void on_namespaceUpdateButton_clicked(GtkWidget * dummy1,
   } else {
     EncName updateName;
 
-    if (OK != FSUI_computeNextId(list->name,
-				 &prevId,
-				 &cls.thisId,
-				 cls.updateInterval,
-				 &nextId)) {
-      BREAK();
+    if (OK != NS_computeNextId(ectx,
+			       cfg,
+			       list->name,
+			       &prevId,
+			       &cls.thisId,
+			       cls.updateInterval,
+			       &nextId)) {
+      GE_BREAK(ectx, 0);
       UNREF(metaXML);
       metaXML = NULL;
+      if (last != NULL)
+	free(last);
+      if (next != NULL)
+	free(next);
+      if (freq != NULL)
+	free(freq);
       return;
     }
     hash2enc(&nextId,
@@ -1022,6 +1107,7 @@ void on_namespaceUpdateButton_clicked(GtkWidget * dummy1,
 				   "namespaceUpdateIntervalComboBoxEntry",
 				   &cls.updateInterval)) {
       error = _("Failed to parse given time interval!");
+      identifierName = ""; /* to make gcc happy */
     } else {
       identifierName = gtk_entry_get_text(GTK_ENTRY(nameLine));
       if ( (cls.updateInterval != ECRS_SBLOCK_UPDATE_NONE) &&
@@ -1042,6 +1128,9 @@ void on_namespaceUpdateButton_clicked(GtkWidget * dummy1,
 	 error);
       gtk_dialog_run(GTK_DIALOG(dialog));
       gtk_widget_destroy(dialog);
+      FREENONNULL(last);
+      FREENONNULL(next);
+      FREENONNULL(freq);
       return;
     }
     hash(identifierName,
@@ -1056,7 +1145,7 @@ void on_namespaceUpdateButton_clicked(GtkWidget * dummy1,
     cls.anonymityLevel
       = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin));
 
-    gtk_tree_selection_selected_foreach
+    ggc_tree_selection_selected_foreach
       (selection,
        &initiateUpload,
        &cls);
@@ -1065,47 +1154,213 @@ void on_namespaceUpdateButton_clicked(GtkWidget * dummy1,
   gtk_widget_destroy(dialog);
   UNREF(metaXML);
   metaXML = NULL;
+  if (last != NULL)
+    free(last);
+  if (next != NULL)
+    free(next);
+  if (freq != NULL)
+    free(freq);
+  DEBUG_END();
 }
 
-void on_clearAvailableContentButton_clicked(GtkWidget * dummy1,
-					    GtkWidget * dummy2) {
+void on_clearAvailableContentButton_clicked_fs(GtkWidget * dummy1,
+					       GtkWidget * dummy2) {
   GtkWidget * contentList;
   GtkTreeModel * model;
 
+  DEBUG_BEGIN();
   contentList
     = glade_xml_get_widget(getMainXML(),
 			   "availableContentList");
   model
     = gtk_tree_view_get_model(GTK_TREE_VIEW(contentList));
-  FSUI_clearTrackedURIS();
+  URITRACK_clearTrackedURIS(ectx,
+			    cfg);
   gtkSaveCall(&clearContentList, model);
-  updateContentList(NULL);
+  DEBUG_END();
 }
 
-void on_trackingCheckButton_toggled(GtkWidget * dummy1,
-				    GtkWidget * dummy2) {
+void on_trackingCheckButton_toggled_fs(GtkWidget * dummy1,
+				       GtkWidget * dummy2) {
   GtkWidget * trackCheckButton;
 
   trackCheckButton
     = glade_xml_get_widget(getMainXML(),
 			   "trackingCheckButton");
-  FSUI_trackURIS(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(trackCheckButton)) == TRUE ?
-		 YES : NO);
+  URITRACK_trackURIS(ectx,
+		     cfg,
+		     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(trackCheckButton)) == TRUE ?
+		     YES : NO);
 }
 
+/**
+ * The spin button giving the rating for a particular namespace
+ * has been changed.  Store the new rating for the namespace.
+ */
+void on_namespaceRatingSpinButton_changed_fs(GtkWidget * dummy,
+					     GtkWidget * dummy2) {
+  GtkWidget * spin;
+  GtkWidget * ncbe;
+  GtkTreeModel * model;
+  GtkTreeIter iter;
+  char * encStr;
+  char * description;
+  int rating;
+  int newrating;
+
+  DEBUG_BEGIN();
+  spin
+    = glade_xml_get_widget(getMainXML(),
+			   "namespaceRatingSpinButton");
+  ncbe
+    = glade_xml_get_widget(getMainXML(),
+			   "searchNamespaceComboBoxEntry");
+  model = gtk_combo_box_get_model(GTK_COMBO_BOX(ncbe));
+  description = NULL;
+  encStr = NULL;
+  if (TRUE == gtk_combo_box_get_active_iter(GTK_COMBO_BOX(ncbe),
+					    &iter)) {
+    gtk_tree_model_get(model,
+		       &iter,
+		       NS_SEARCH_DESCRIPTION, &description,
+		       NS_SEARCH_ENCNAME, &encStr,
+		       NS_SEARCH_RATING, &rating,
+		       -1);
+    if ( (description != NULL) &&
+	 (0 == strcmp(description,
+		      _("globally"))) ) {
+      /* just to be sure */
+      gtk_widget_set_sensitive(spin,
+			       FALSE);
+    } else {
+      if (encStr != NULL) {
+	newrating = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spin));
+	rating = NS_rankNamespace(ectx,
+				  cfg,
+				  encStr,
+				  newrating - rating);
+	if (rating != newrating) {
+	  /* concurrent modification? */
+	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin),
+				    rating);
+	  GE_BREAK(ectx, 0);	
+	}
+	gtk_list_store_set(GTK_LIST_STORE(model),
+			   &iter,
+			   NS_SEARCH_RATING, rating,
+			   -1);
+      }
+    }
+  } else {
+    /* FIXME: if enc2hash succeeds, we may want to keep this
+       active */
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin),
+			      0);
+    gtk_widget_set_sensitive(spin,
+			     FALSE);
+  }
+  if (description != NULL)
+    free(description);
+  if (encStr != NULL)
+    free(encStr);
+  DEBUG_END();
+}
+
+
+/**
+ * The namespace in the search window has changed.
+ * Update the trust level (possibly changing sensitivity)
+ * and set the search string to the root (if available).
+ */
+void on_searchNamespaceComboBoxEntry_changed_fs(GtkWidget * dummy,
+						GtkWidget * dummy2) {
+  GtkWidget * keyword;
+  GtkWidget * spin;
+  GtkWidget * ncbe;
+  GtkTreeModel * model;
+  GtkTreeIter iter;
+  int rating;
+  char * encStr;
+  char * descStr;
+  HashCode512 ns;
+  HashCode512 root;
+  EncName enc;
+
+  DEBUG_BEGIN();
+  spin
+    = glade_xml_get_widget(getMainXML(),
+			   "namespaceRatingSpinButton");
+  ncbe
+    = glade_xml_get_widget(getMainXML(),
+			   "searchNamespaceComboBoxEntry");
+  model = gtk_combo_box_get_model(GTK_COMBO_BOX(ncbe));
+  descStr = NULL;
+  encStr = NULL;
+  if (TRUE == gtk_combo_box_get_active_iter(GTK_COMBO_BOX(ncbe),
+					    &iter)) {
+    gtk_tree_model_get(model,
+		       &iter,
+		       NS_SEARCH_DESCRIPTION, &descStr,
+		       NS_SEARCH_ENCNAME, &encStr,
+		       NS_SEARCH_RATING, &rating,
+		       -1);
+    if ( (descStr != NULL) &&
+	 (0 == strcmp(descStr,
+		      _("globally"))) ) {
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin),
+				0);
+      gtk_widget_set_sensitive(spin,
+			       FALSE);
+    } else if (encStr != NULL) {
+      enc2hash(encStr,
+	       &ns);
+      gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin),
+				rating);
+      gtk_widget_set_sensitive(spin,
+			       TRUE);
+      if (OK == NS_getNamespaceRoot(ectx,
+				    cfg,
+				    encStr,
+				    &root)) {
+	hash2enc(&root,
+		 &enc);
+	keyword
+	  = glade_xml_get_widget(getMainXML(),
+				 "fssearchKeywordComboBoxEntry");
+	gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(keyword))),
+			   (const gchar*) &enc);
+      }
+    }
+  } else {
+    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin),
+			      0);
+    gtk_widget_set_sensitive(spin,
+			     FALSE);
+  }
+  if (descStr != NULL)
+    free(descStr);
+  if (encStr != NULL)
+    free(encStr);
+  DEBUG_END();
+}
+
+
 
 void fs_namespace_start() {
   GtkWidget * contentList;
   GtkListStore * model;
   GtkCellRenderer * renderer;
   GtkWidget * trackCheckButton;
+  GtkTreeViewColumn * column;
   int col;
 
+  DEBUG_BEGIN();
   trackCheckButton
     = glade_xml_get_widget(getMainXML(),
 			   "trackingCheckButton");
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(trackCheckButton),
-			       FSUI_trackStatus() == YES ? TRUE : FALSE);
+			       URITRACK_trackStatus(ectx,
+						    cfg) == YES ? TRUE : FALSE);
 
   contentList
     = glade_xml_get_widget(getMainXML(),
@@ -1114,11 +1369,14 @@ void fs_namespace_start() {
   model = gtk_list_store_new(NAMESPACE_NUM,
 			     G_TYPE_STRING, /* name */
 			     G_TYPE_UINT64, /* size */
+			     G_TYPE_STRING, /* human-readable size */
 			     G_TYPE_STRING, /* uri-string */
 			     G_TYPE_POINTER,
 			     G_TYPE_POINTER); /* uri */
   gtk_tree_view_set_model(GTK_TREE_VIEW(contentList),
 			  GTK_TREE_MODEL(model));
+  gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(contentList)),
+			      GTK_SELECTION_MULTIPLE);
   renderer = gtk_cell_renderer_text_new();
   col = gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(contentList),
 					      -1,
@@ -1126,16 +1384,32 @@ void fs_namespace_start() {
 					      renderer,
 					      "text", NAMESPACE_FILENAME,
 					      NULL);
+  column = gtk_tree_view_get_column(GTK_TREE_VIEW(contentList),
+				    col - 1);
+  gtk_tree_view_column_set_resizable(column, TRUE);
+  gtk_tree_view_column_set_clickable(column, TRUE);
+  gtk_tree_view_column_set_reorderable(column, TRUE);
+  gtk_tree_view_column_set_sort_column_id(column, NAMESPACE_FILENAME);
+  /*gtk_tree_view_column_set_sort_indicator(column, TRUE);*/
+
   gtk_tree_view_column_set_resizable(gtk_tree_view_get_column(GTK_TREE_VIEW(contentList),
                                               col - 1),
                                               TRUE);
   renderer = gtk_cell_renderer_text_new();
+  g_object_set (renderer, "xalign", 1.00, NULL);
   col = gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(contentList),
 					      -1,
 					      _("Filesize"),
 					      renderer,
-					      "text", NAMESPACE_SIZE,
+					      "text", NAMESPACE_HSIZE,
 					      NULL);
+  column = gtk_tree_view_get_column(GTK_TREE_VIEW(contentList),
+				    col - 1);
+  gtk_tree_view_column_set_resizable(column, TRUE);
+  gtk_tree_view_column_set_clickable(column, TRUE);
+  gtk_tree_view_column_set_reorderable(column, TRUE);
+  gtk_tree_view_column_set_sort_column_id(column, NAMESPACE_SIZE);
+  /*gtk_tree_view_column_set_sort_indicator(column, TRUE);*/
   gtk_tree_view_column_set_resizable(gtk_tree_view_get_column(GTK_TREE_VIEW(contentList),
                                               col - 1),
                                               TRUE);
@@ -1146,28 +1420,31 @@ void fs_namespace_start() {
 					      renderer,
 					      "text", NAMESPACE_URISTRING,
 					      NULL);
+  column = gtk_tree_view_get_column(GTK_TREE_VIEW(contentList),
+				    col - 1);
+  gtk_tree_view_column_set_reorderable(column, TRUE);
+  gtk_tree_view_column_set_resizable(column, TRUE);
   gtk_tree_view_column_set_resizable(gtk_tree_view_get_column(GTK_TREE_VIEW(contentList),
-                                              col - 1),
-                                              TRUE);
-  addCronJob(&updateContentList,
-	     0,
-	     UPDATE_CONTENT_LIST_FREQUENCY,
-	     NULL);
-  FSUI_listNamespaces(ctx,
-		      YES,
-		      &addTabForNamespace,
-		      NULL);
+							      col - 1),
+				     TRUE);
+  URITRACK_registerTrackCallback(ectx,
+				 cfg,
+				 &updateViewSave,
+				 NULL);
+  NS_listNamespaces(ectx,
+		    cfg,
+		    YES,
+		    &addTabForNamespace,
+		    NULL);
+  DEBUG_END();
 }
 
 #if 0
-void on_availableContentList_destroy(GtkWidget * dummy1,
-				     GtkWidget * dummy2) {
+void on_availableContentList_destroy_fs(GtkWidget * dummy1,
+					GtkWidget * dummy2) {
   GtkWidget * contentList;
   GtkTreeModel * model;
 
-  delCronJob(&updateContentList,
-	     UPDATE_CONTENT_LIST_FREQUENCY,
-	     NULL);
   contentList
     = glade_xml_get_widget(getMainXML(),
 			   "availableContentList");
@@ -1176,8 +1453,8 @@ void on_availableContentList_destroy(GtkWidget * dummy1,
   clearContentList(model);
 }
 
-void on_localNamespacesNotebook_destroy(GtkWidget * dummy1,
-					GtkWidget * dummy2) {
+void on_localNamespacesNotebook_destroy_fs(GtkWidget * dummy1,
+					   GtkWidget * dummy2) {
   NamespaceList * pos;
   GtkTreeIter iter;
   struct ECRS_URI * u;
@@ -1215,6 +1492,9 @@ void on_localNamespacesNotebook_destroy(GtkWidget * dummy1,
 #endif
 
 void fs_namespace_stop() {
+  URITRACK_unregisterTrackCallback(&updateViewSave,
+				   NULL);
+  /* FIXME: free resources! */
 }
 
 /* end of namespace.c */
