@@ -45,14 +45,13 @@ typedef struct SE_ {
   UpdateData getData;
   void * get_closure;
   unsigned int count;
-  int fill; /* YES / NO */
+  int fill; /* fill first "fill" entries */
 } StatEntry;
 
 extern StatEntry stats[];
 
-extern int UPDATE_INTERVAL;
-
-void init_functions(void);
+void init_functions(struct GE_Context * e,
+		    struct GC_Configuration * c);
 
 void done_functions(void);
 
